/*

=head1 NAME

FlickrTags.js - display flickr.com images associated with one, or more,
topics defined in a webpage's meta headers.

=head1 SYNOPSIS

 [html]
  [head]
   [title]...[/title]
   [meta  name="des" content="Elections;Demonstrations and Riots"]
   [meta  name="per" content=""]
   [meta  name="org" content=""]
   [meta  name="geo" content="Ukraine"]
   [!-- ... --]
   [script type = "text/javascript" src = "foo.js"]
    //
   [/script]
  [/head]
  [body]
   [!-- ... --]
   [script]
    var flickr = new FlickrTags ();
    flickr.add_topic("geo");
    flickr.draw();
   [/script]
  [/body]
 [/html]

=head1 DESCRIPTION

FlickrTags.js will redraw the current DOM to display flickr.com images
associated with one, or more, topics defined in a webpage's meta
headers. The list of headers to poll is user-defined.

=cut

=head1 FlickrTags PACKAGE METHODS

=cut

=head2 new FlickrTags ()

Returns a I<FlickrTagsg> object.

=cut

*/

function FlickrTags () {
    this._topics = new Array();
}

/*

=head1 FlickrTags OBJECT METHODS

=cut

=head2 foo.add_topic(string)

Add a topic to search for tags that are polled from flickr.com.

The value of I<string> will be compared against the B<name> attribute
of each meta tag. If there is a match, then tags will be assigned
using the semi-colon (;) separated list of values defined by the
B<content> attribute.

=cut

*/

FlickrTags.prototype.add_topic = function (topic) {
    this._topics.push(topic);
}

/*

=head2 foo.draw()

Redraw the DOM, inserting a wrapper containing images as the B<first>
child of the body element.

The wrapper, and its children, are assigned the following CSS classes :

=over 4

=item * B<ftag_flickr>

Assigned to a [div] element that wraps images
and errors, if any.

=item * B<ftag_imgwrapper>

Assigned to a [div] element that wraps each image.

=item * B<ftag_img>

Assigned to each [img] element.

=item * B<ftag_errors>

Assigned to a [div] element that wraps an unordered 
list of errors.

=back

Returns true or false.

=cut

*/

FlickrTags.prototype.draw = function () {

    var url_flickr = "http://www.flickr.com/";
    var url_photos = url_flickr + "photos/";
    var url_tags   = url_flickr + "services/feeds/photos_public.gne";

    var txt_warn = "errors";

    var css_flickr  = "border:1px solid #ccc;padding:10px;margin-right:20px;"
    var css_wrapper = "height:100px;width:100px;margin:5px;float:left;";
    var css_img     = "border:none;margin:5px;"
    var css_errors  = "color:red;";

    var regexp_meta   = new RegExp("^(" + this._topics.join("|") + ")$");
    var regexp_photos = new RegExp("/((?:[0-9]+)_(?:[0-9a-z]+))_m\.jpg");

    var topics   = new Object;
    var keywords = new Object;
    var images   = new Object;
    var links    = new Object;
    var errors   = new Array;

    var useragent = new XMLHttpRequest ();

    if (! useragent) {
	return false;
    }

    var meta = document.getElementsByTagName("meta");

    for (var i = 0; i < meta.length; i++) {

         if (regexp_meta.test(meta[i].getAttribute("name"))) {

	     var content = meta[i].getAttribute("content").split(";");

	     for (j = 0; j < content.length; j++) {
                var kw = content[j];
                if (kw) {
		   topics[kw] ++; 
	        }
	     }
	 }
    }

    //

    for (kw in topics) {			

	url_atom  = url_tags + "?tags=" + kw + "&format=atom_03";

	useragent.open("GET",url_atom,false);

        try {			     
           useragent.send(null);
        } 
			     
        catch (e) {
            errors.append("failed to retrieve " + url_atom + "," + e);
            continue;
	}

        if (useragent.status != 200) {
           errors.append("failed to retrieve " + url_atom + ", " + user.statusText);   
           continue;
        }

	doc = useragent.responseXML;

        if (! doc) {
            errors.append("failed to build XML Document for " + url_atom);
            continue;
        }

        entries = doc.getElementsByTagName("entry");

	for (var i = 0; i < entries.length; i++) {

            var link    = entries[i].getElementsByTagName("link")[0].getAttribute("href");
            var content = entries[i].getElementsByTagName("content")[0].childNodes[0];
	    var html    = content.nodeValue;

	    var matches = regexp_photos.exec(html);

	    if (! matches) {
                 errors.append("failed to locate photo for entry " + link + " (" + url_atom + ")");
                 continue;
	    }

            var match = matches[1];

            if (! match) {
                 errors.append("failed to locate photo for entry " + link + " (" + url_atom + ")");
                 continue;
	    }

	    images[match] ++;
            links[match] = link;

            if (! keywords[match]) {
                keywords[match] = new Array();
            }

            keywords[match].push();			    
        }
    }

    //			    

    flickr = document.createElement("div");
    flickr.setAttribute("class","ftag_flickr");

    for (id in images) {
       var url_thumb = url_photos + id + "_t.jpg";

       var wrapper = document.createElement("div");
       wrapper.setAttribute("class","ftag_imgwrapper");
			    
       var link = document.createElement("a");
       link.setAttribute("href",links[id]);
       link.setAttribute("alt",keywords[id].join(";"));
       
       var img = document.createElement("img");
       img.setAttribute("src",url_thumb);
       img.setAttribute("class","ftag_img");

       link.appendChild(img);		
       wrapper.appendChild(link);

       flickr.insertBefore(wrapper,flickr.firstChild);		
    }			

    //

    var cl = document.createElement("div");
    cl.setAttribute("style","clear:both;");
    flickr.appendChild(cl);

    //

    var errs = errors.length;
			    
    if (errs) {
         var warn = document.createElement("div");
	 warn.setAttribute("class","ftag_errors");

         var msg  = document.createElement("p");
         var list = document.createElement("ul");

         for (var i = 0; i < errs; i++) {
             var item = document.createElement("li");

             item.appendChild(document.createTextElement(errors[i]));
             list.appendChild(item);			     
         }

         msg.appendChild(document.createTextElement(txt_warn));

         warn.appendChild(msg);    
         warn.appendChild(list);

	 flickr.insertBefore(warn,flickr.firstChild);
    }

    //
			     
    void(document.body.insertBefore(flickr,document.body.firstChild));
    return true;
}

/*

netscape.security.PrivilegeManager.enablePrivilege('UniversalBrowserRead');

var flickr = new FlickrTags ();
flickr.add_topic("geo");
flickr.add_topic("des");
flickr.add_topic("per");
flickr.add_topic("org");
flickr.draw();

*/

/*

=head1 VERSION

1.0

=head1 DATE

$Date: 2004/12/06 17:54:05 $

=head1 AUTHOR

Aaron Straup Cope E<lt>www.aaronstraupcope.comE<gt>

=head1 LICENSE

Copyright (c) 2004 Aaron Straup Cope. All rights reserved.

This is free software, you may use it and distribute it under
the same terms as the Perl Artistic License.

=cut

$Id: flickr-tags.js,v 1.4 2004/12/06 17:54:05 asc Exp $

*/
